@@ -8,7 +8,7 @@ use Carp qw(croak);
 use AnyEvent;
 use AnyEvent::Handle;
 
-our $VERSION = 0.6;
+our $VERSION = 0.7;
 
 =head1 NAME
 
@@ -20,9 +20,9 @@ AnyEvent::STOMP - A lightweight event-driven STOMP client
  use AnyEvent::STOMP;
 
  my $client = AnyEvent::STOMP->connect($host, $port, $ssl, $destination, $ack,
-                                       { connect_headers }, 
+                                       { connect_headers },
                                        { subscribe_headers });
- 
+
  $client->send($command, $headers, $body);
 
  # Register interest in new messages
@@ -45,18 +45,18 @@ AnyEvent::STOMP - A lightweight event-driven STOMP client
 =head1 DESCRIPTION
 
 AnyEvent::STOMP is a lightweight event-driven STOMP client.  It's intended
-to be run in the context of an event loop driven by AnyEvent.  
+to be run in the context of an event loop driven by AnyEvent.
 
 =head2 Making a connection
 
  my $client = AnyEvent::STOMP->connect($host, $port, $ssl, $destination, $ack,
-                                       { connect_headers }, 
+                                       { connect_headers },
                                        { subscribe_headers });
 
 Only the first parameter (the hostname) is required.  The remaining optional
 arguments are:
 
-=over 
+=over
 
 =item port
 
@@ -73,7 +73,7 @@ If defined, subscribe to the specified destination (queue) upon connection.
 
 =item ack
 
-Sets the behavior with respect to STOMP frame acknowledgments.  
+Sets the behavior with respect to STOMP frame acknowledgments.
 
 If this value is 0 or undef, no acknowledgment is required: the server will
 consider all sent frames to be delivered, regardless of whether the client has
@@ -81,9 +81,9 @@ actually received them.  (This is the default behavior according to the STOMP
 protocol.)
 
 If set to C<auto>, the client will automatically acknowledge a frame upon
-receipt.  
+receipt.
 
-If set to C<manual>, the caller must acknowledge frames manually via the 
+If set to C<manual>, the caller must acknowledge frames manually via the
 ack() method.
 
 =item connect_headers
@@ -100,19 +100,19 @@ frame.
 
 =cut
 
-sub connect { 
-    my $class = shift; 
-    my ($host, $port, $ssl, $destination, $ack, 
+sub connect {
+    my $class = shift;
+    my ($host, $port, $ssl, $destination, $ack,
         $connect_headers, $subscribe_headers) = @_;
 
     croak 'No host provided' unless $host;
-    croak "ack value must be 0, undef, 'auto' or 'manual'" 
+    croak "ack value must be 0, undef, 'auto' or 'manual'"
         if $ack && $ack ne 'auto' && $ack ne 'manual';
 
     my $self = $class->SUPER::new;
 
     $self->{ack} = $ack;
-    
+
     $port ||= ($ssl ? 61612 : 61613);
 
     my $connect_cb;
@@ -121,15 +121,15 @@ sub connect {
         tls => $ssl ? 'connect' : undef,
         keepalive => 1,
         on_prepare => sub { $self->event('prepare', @_); },
-        on_connect => sub { 
+        on_connect => sub {
             $self->event('connect', @_);
             $self->send_frame('CONNECT', undef, $connect_headers);
             if ($destination) {
                 $subscribe_headers->{destination} = $destination;
                 $subscribe_headers->{ack} = 'client' if $ack;
-                $connect_cb = $self->reg_cb(CONNECTED => sub { 
+                $connect_cb = $self->reg_cb(CONNECTED => sub {
                         $self->{session_id} = $_[2]->{session};
-                        $self->send_frame('SUBSCRIBE', 
+                        $self->send_frame('SUBSCRIBE',
                             undef, $subscribe_headers);
                         undef $connect_cb;
                 });
@@ -151,7 +151,7 @@ sub connect {
 =head2 Sending a message
 
 To send a message, just call send() with the body, the destination (queue)
-name, and (optionally) any additional headers:  
+name, and (optionally) any additional headers:
 
  $client->send($body, $destination, $headers); # headers may be undef
 
@@ -176,6 +176,26 @@ You can also send arbitrary STOMP frames:
 See the STOMP protocol documentation for more details on valid commands and
 headers.
 
+=head3 Content Length
+
+The C<content-length> header is special because it is sometimes used to
+indicate the length of the body but also the JMS type of the message in
+ActiveMQ as per L<http://activemq.apache.org/stomp.html>.
+
+If you do not supply a C<content-length> header, following the protocol
+recommendations, a C<content-length> header will be added if the frame has a
+body.
+
+If you do supply a numerical C<content-length> header, it will be used as
+is. Warning: this may give unexpected results if the supplied value does not
+match the body length. Use only with caution!
+
+Finally, if you supply an empty string as the C<content-length> header, it
+will not be sent, even if the frame has a body. This can be used to mark a
+message as being a TextMessage for ActiveMQ. Here is an example of this:
+
+ $client->send_frame($command, $body, { 'content-length' => '' } );
+
 =cut
 
 sub send_frame {
@@ -184,11 +204,16 @@ sub send_frame {
 
     croak 'Missing command' unless $command;
 
-    $headers->{'content-length'} = length $body;
+    my $tmp = $headers->{'content-length'};
+    if (!defined $tmp) {
+        $headers->{'content-length'} = length $body;
+    } elsif ($tmp eq '') {
+        delete $headers->{'content-length'};
+    }
 
-    my $frame = sprintf("%s\n%s\n\n%s\000",
+    my $frame = sprintf("%s\n%s\n%s\000",
                         $command,
-                        join("\n", map { "$_:$headers->{$_}" } keys %$headers),
+                        join('', map { "$_:$headers->{$_}\n" } keys %$headers),
                         $body);
 
     $self->{handle}->push_write($frame);
@@ -196,14 +221,14 @@ sub send_frame {
 
 =head2 Events
 
-Once you've connected, you can register interest in events, most commonly 
-the receipt of new messages (assuming you've connected as a subscriber).  
+Once you've connected, you can register interest in events, most commonly
+the receipt of new messages (assuming you've connected as a subscriber).
 
 The typical use is:
 
  $client->reg_cb($type => $cb->($client, $body, $headers));
 
-In most cases, $type is C<MESSAGE>, but you can also register interest 
+In most cases, $type is C<MESSAGE>, but you can also register interest
 in any other type of frame (C<RECEIPT>, C<ERROR>, etc.).  (If you register
 interest in C<CONNECTED> frames, please do so with a priority of C<after>;
 see Object::Event for more details.)
@@ -227,7 +252,7 @@ See C<on_connect> in AnyEvent::Handle for more details.
 
 =item frame => $cb->($client, $type, $body, $headers)
 
-Will be fired whenever any frame is received.  
+Will be fired whenever any frame is received.
 
 =item connect_error => $cb->($client, $errmsg)
 
@@ -236,7 +261,7 @@ C<on_connect_error> in AnyEvent::Handle for more details.
 
 =item io_error => $cb->($client, $errmsg)
 
-Will be fired if an I/O error occurs.  See C<on_error> in AnyEvent::Handle 
+Will be fired if an I/O error occurs.  See C<on_error> in AnyEvent::Handle
 for more details.
 
 =back
@@ -257,8 +282,8 @@ sub _receive_frame {
                 $command = $1;
             }
             foreach my $line (split(/\n/, $raw_headers)) {
-                my ($key, $value) = split(/\s*:\s*/, $line, 2);                                                       
-                $headers->{$key} = $value;                                                                   
+                my ($key, $value) = split(/\s*:\s*/, $line, 2);
+                $headers->{$key} = $value;
             }
             my @args;
             if (my $content_length = $headers->{'content-length'}) {
@@ -266,12 +291,12 @@ sub _receive_frame {
             } else {
                 @args = ('regex' => qr/.*?\000\n*/s);
             }
-            $self->{handle}->unshift_read(@args, sub { 
+            $self->{handle}->unshift_read(@args, sub {
                     $body = $_[1];
                     $body =~ s/\000\n*$//;
 
                     if ($self->{ack} eq 'auto' && defined $headers->{'message-id'}) {
-                        $self->send_frame('ACK', undef, 
+                        $self->send_frame('ACK', undef,
                                           {'message-id' => $headers->{'message-id'}});
                     }
 
@@ -303,8 +328,20 @@ sub ack {
     $self->send_frame('ACK', undef, $headers);
 }
 
-sub DESTROY {
-    shift->send('DISCONNECT');
+=head2 Closing a session
+
+When done with a session, you need to explicitly call the destroy() method.
+It will also send a DISCONNECT message on your behalf before closing.
+Attempting to let the object fall out-of scope is not sufficient.
+
+  $client->destroy;
+
+=cut
+
+sub destroy {
+	my ($self) = shift;
+    $self->send_frame('DISCONNECT') if defined undef $self->{handle};
+	undef $self->{handle};
 }
 
 =head1 SEE ALSO
@@ -314,12 +351,14 @@ L<http://stomp.codehaus.org/Protocol>
 
 =head1 AUTHORS AND CONTRIBUTORS
 
-Michael S. Fischer (L<michael+cpan@dynamine.net>) wrote the original version
-and is the current maintainer.
+Fulko.Hew (L<fulko.hew@gmail.com>) is the current maintainer.
+
+Michael S. Fischer (L<michael+cpan@dynamine.net>) wrote the original version.
 
 =head1 COPYRIGHT AND LICENSE
 
-(C) 2010 Yahoo! Inc. 
+(C) 2014 SITA INC Canada, Inc.
+(C) 2010 Yahoo! Inc.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
